"""
A completed test script for the recursive functions.

Author: Walker M. White
Date: October 12, 2020
"""
import code
import seg
import person
import family
import introcs

def test_decode():
    """
    Test procedure for the function decode()
    """
    print('Testing decode()')

    result = code.decode([])
    introcs.assert_equals('',result)

    result = code.decode([['b',1]])
    introcs.assert_equals('b',result)

    result = code.decode([['a',3]])
    introcs.assert_equals('aaa',result)

    result = code.decode([['a',3],['h',1],['a',1]])
    introcs.assert_equals('aaaha',result)


def test_encode():
    """
    Test procedure for the function encode()
    """
    print('Testing encode()')

    result = code.encode('')
    introcs.assert_equals([],result)

    result = code.encode('b')
    introcs.assert_equals([['b',1]],result)

    result = code.encode('aaa')
    introcs.assert_equals([['a',3]],result)

    result = code.encode('aaaha')
    introcs.assert_equals([['a',3],['h',1],['a',1]],result)


def test_segregate():
    """
    Test the function segregate
    """
    print('Testing segregate()')

    result = seg.segregate([])
    introcs.assert_equals((-1, []),result)

    result = seg.segregate([-1])
    introcs.assert_equals((-1, [-1]),result)

    result = seg.segregate([5])
    introcs.assert_equals((0, [5]),result)

    result = seg.segregate([1, -1, 2, -5, -3, 0])
    introcs.assert_equals((3, [-1, -5, -3, 1, 2, 0]), result)

    result = seg.segregate([-1, -5, -3])
    introcs.assert_equals((-1, [-1, -5, -3]),result)

    result = seg.segregate([1, 5, 3])
    introcs.assert_equals((0, [1, 5, 3]),result)




def test_ancestors():
    """
    Test the function ancestors
    """
    print('Testing ancestors()')

    # GREAT GRANDPARENTS (some unknown)
    # John Smith Sr.
    ggd1 = person.Person('John', 'Smith')
    ggm1 = person.Person('Pamela', 'Grey')

    ggm2 = person.Person('Eva', 'Brown')

    ggd4 = person.Person('Dan', 'O\'Reilly')
    ggm4 = person.Person('Heather', 'Chase')

    # GRANDPARENTS
    # John Smith Jr.
    gd1 = person.Person('John','Smith',ggm1,ggd1)
    gm1 = person.Person('Jane','Dare',ggm2,None)

    gd2 = person.Person('John','Evans')
    gm2 = person.Person('Ellen','O\'Reilly',ggm4,ggd4)

    # PARENTS
    # John Smith III
    d = person.Person('John','Smith',gm1,gd1)
    m = person.Person('Pamela','Evans',gm2,gd2)

    # FINAL GENERATION
    # John Smith IV
    p = person.Person('John','Smith',m,d)

    result = family.ancestors(ggd1)
    introcs.assert_equals([], result)

    result = family.ancestors(gm1)
    introcs.assert_equals(['Eva'], result)

    result = family.ancestors(gm2)
    introcs.assert_equals(['Dan', 'Heather'], result)

    result = family.ancestors(d)
    introcs.assert_equals(['Eva', 'Jane', 'John', 'John', 'Pamela'], result)

    result = family.ancestors(p)
    correct = ['Dan', 'Ellen', 'Eva', 'Heather', 'Jane', 'John', 'John', 'John', 'John', 'Pamela', 'Pamela']
    introcs.assert_equals(correct, result)


def test_related():
    """
    Test the function related
    """
    print('Testing ancestor()')

    # GREAT GRANDPARENTS (some unknown)
    # John Smith Sr.
    ggd1 = person.Person('John', 'Smith')
    ggm1 = person.Person('Pamela', 'Grey')

    ggm2 = person.Person('Eva', 'Brown')

    ggd4 = person.Person('Dan', 'O\'Reilly')
    ggm4 = person.Person('Heather', 'Chase')

    # GRANDPARENTS
    # John Smith Jr.
    gd1 = person.Person('John','Smith',ggm1,ggd1)
    gm1 = person.Person('Jane','Dare',ggm2,None)

    gd2 = person.Person('John','Evans')
    gm2 = person.Person('Ellen','O\'Reilly',ggm4,ggd4)

    # PARENTS (and UNCLE, AUNT)
    # John Smith III
    d = person.Person('John','Smith',gm1,gd1)
    m = person.Person('Pamela','Evans',gm2,gd2)
    a = person.Person('Alice','Smith',gm1,gd1)
    u = person.Person('Roger','Evans',gm2,gd2)

    # FINAL GENERATION
    # John Smith IV
    p = person.Person('John','Smith',m,d)
    q = person.Person('Tiffany','Smith',m,d)

    result = family.related(p,None)
    introcs.assert_equals(False, result)

    result = family.related(None,q)
    introcs.assert_equals(False, result)

    result = family.related(p,p)
    introcs.assert_equals(True, result)

    result = family.related(p,q)
    introcs.assert_equals(True, result)

    result = family.related(p,u)
    introcs.assert_equals(True, result)

    result = family.related(p,a)
    introcs.assert_equals(True, result)

    result = family.related(a,u)
    introcs.assert_equals(False, result)

    result = family.related(p,ggm1)
    introcs.assert_equals(True, result)


if __name__ == '__main__':
    test_decode()
    test_encode()
    test_segregate()
    test_ancestors()
    test_related()
    print('The recursive functions passed all tests.')
